@@ -16,7 +16,7 @@
 # under the License.
 
 #' @include enums.R
-#' @include arrow-package.R
+#' @include arrow-object.R
 #' @include io.R
 
 #' @title Compression Codec class
@@ -38,7 +38,8 @@
 #' @rdname Codec
 #' @name Codec
 #' @export
-Codec <- R6Class("Codec", inherit = ArrowObject,
+Codec <- R6Class("Codec",
+  inherit = ArrowObject,
   active = list(
     name = function() util___Codec__name(self),
     level = function() abort("Codec$level() not yet implemented")
@@ -60,17 +61,17 @@ Codec$create <- function(type = "gzip", compression_level = NA) {
 #' the Arrow C++ library. This function lets you know which are available for
 #' use.
 #' @param type A string, one of "uncompressed", "snappy", "gzip", "brotli",
-#' "zstd", "lz4", "lzo", or "bz2", case insensitive.
+#' "zstd", "lz4", "lzo", or "bz2", case-insensitive.
 #' @return Logical: is `type` available?
 #' @export
-#' @examplesIf arrow_available()
+#' @examples
 #' codec_is_available("gzip")
 codec_is_available <- function(type) {
   util___Codec__IsAvailable(compression_from_name(type))
 }
 
 compression_from_name <- function(name) {
-  map_int(name, ~CompressionType[[match.arg(toupper(.x), names(CompressionType))]])
+  map_int(name, ~ CompressionType[[match.arg(toupper(.x), names(CompressionType))]])
 }
 
 #' @title Compressed stream classes
@@ -97,7 +98,7 @@ compression_from_name <- function(name) {
 #'
 #' Methods are inherited from [InputStream] and [OutputStream], respectively
 #' @export
-#' @include arrow-package.R
+#' @include arrow-object.R
 CompressedOutputStream <- R6Class("CompressedOutputStream", inherit = OutputStream)
 CompressedOutputStream$create <- function(stream, codec = "gzip", compression_level = NA) {
   codec <- Codec$create(codec, compression_level = compression_level)
