@@ -80,14 +80,16 @@ arrow::Result<std::shared_ptr<arrow::DataType>> RExtensionType::Deserialize(
   // an event loop from wherever this *might* be called is high and hard to
   // predict. As a compromise, just create the instance when it is safe to
   // do so.
-  if (GetMainRThread().IsMainThread()) {
+  if (MainRThread::GetInstance().IsMainThread()) {
     r6_instance();
   }
 
   return std::shared_ptr<RExtensionType>(cloned.release());
 }
 
-std::string RExtensionType::ToString() const {
+std::string RExtensionType::ToString() const { return ToString(false); }
+
+std::string RExtensionType::ToString(bool show_metadata) const {
   arrow::Result<std::string> result = SafeCallIntoR<std::string>([&]() {
     cpp11::environment instance = r6_instance();
     cpp11::function instance_ToString(instance["ToString"]);
@@ -98,7 +100,7 @@ std::string RExtensionType::ToString() const {
   // In the event of an error (e.g., we are not on the main thread
   // and we are not inside RunWithCapturedR()), just call the default method
   if (!result.ok()) {
-    return ExtensionType::ToString();
+    return ExtensionType::ToString(show_metadata);
   } else {
     return result.ValueUnsafe();
   }
