@@ -238,13 +238,6 @@ std::string fs___FileSystem__type_name(
   return file_system->type_name();
 }
 
-// [[arrow::export]]
-std::shared_ptr<fs::LocalFileSystem> fs___LocalFileSystem__create() {
-  // Affects OpenInputFile/OpenInputStream
-  auto io_context = MainRThread::GetInstance().CancellableIOContext();
-  return std::make_shared<fs::LocalFileSystem>(io_context);
-}
-
 // [[arrow::export]]
 std::shared_ptr<fs::SubTreeFileSystem> fs___SubTreeFileSystem__create(
     const std::string& base_path, const std::shared_ptr<fs::FileSystem>& base_fs) {
@@ -265,12 +258,13 @@ std::string fs___SubTreeFileSystem__base_path(
 
 // [[arrow::export]]
 cpp11::writable::list fs___FileSystemFromUri(const std::string& path) {
-  using cpp11::literals::operator"" _nm;
+  using cpp11::literals::operator""_nm;
 
   std::string out_path;
-  return cpp11::writable::list(
-      {"fs"_nm = cpp11::to_r6(ValueOrStop(fs::FileSystemFromUri(path, &out_path))),
-       "path"_nm = out_path});
+  auto io_context = MainRThread::GetInstance().CancellableIOContext();
+  return cpp11::writable::list({"fs"_nm = cpp11::to_r6(ValueOrStop(
+                                    fs::FileSystemFromUri(path, io_context, &out_path))),
+                                "path"_nm = out_path});
 }
 
 // [[arrow::export]]
@@ -295,7 +289,8 @@ std::shared_ptr<fs::S3FileSystem> fs___S3FileSystem__create(
     std::string region = "", std::string endpoint_override = "", std::string scheme = "",
     std::string proxy_options = "", bool background_writes = true,
     bool allow_bucket_creation = false, bool allow_bucket_deletion = false,
-    double connect_timeout = -1, double request_timeout = -1) {
+    bool check_directory_existence_before_creation = false, double connect_timeout = -1,
+    double request_timeout = -1) {
   // We need to ensure that S3 is initialized before we start messing with the
   // options
   StopIfNotOk(fs::EnsureS3Initialized());
@@ -337,6 +332,9 @@ std::shared_ptr<fs::S3FileSystem> fs___S3FileSystem__create(
   s3_opts.allow_bucket_creation = allow_bucket_creation;
   s3_opts.allow_bucket_deletion = allow_bucket_deletion;
 
+  s3_opts.check_directory_existence_before_creation =
+      check_directory_existence_before_creation;
+
   s3_opts.request_timeout = request_timeout;
   s3_opts.connect_timeout = connect_timeout;
 
@@ -425,14 +423,22 @@ std::shared_ptr<fs::GcsFileSystem> fs___GcsFileSystem__Make(bool anonymous,
     gcs_opts.default_metadata = strings_to_kvm(options["default_metadata"]);
   }
 
+  // /// \brief The project to use for creating buckets.
+  // ///
+  // /// If not set, the library uses the GOOGLE_CLOUD_PROJECT environment
+  // /// variable. Most I/O operations do not need a project id, only applications
+  // /// that create new buckets need a project id.
+  if (!Rf_isNull(options["project_id"])) {
+    gcs_opts.project_id = cpp11::as_cpp<std::string>(options["project_id"]);
+  }
+
   auto io_context = MainRThread::GetInstance().CancellableIOContext();
-  // TODO(ARROW-16884): update when this returns Result
-  return fs::GcsFileSystem::Make(gcs_opts, io_context);
+  return ValueOrStop(fs::GcsFileSystem::Make(gcs_opts, io_context));
 }
 
 // [[gcs::export]]
 cpp11::list fs___GcsFileSystem__options(const std::shared_ptr<fs::GcsFileSystem>& fs) {
-  using cpp11::literals::operator"" _nm;
+  using cpp11::literals::operator""_nm;
 
   cpp11::writable::list out;
 
@@ -487,6 +493,10 @@ cpp11::list fs___GcsFileSystem__options(const std::shared_ptr<fs::GcsFileSystem>
     out.push_back({"default_metadata"_nm = metadata});
   }
 
+  if (opts.project_id.has_value()) {
+    out.push_back({"project_id"_nm = opts.project_id.value()});
+  }
+
   return out;
 }
 
