@@ -24,12 +24,13 @@
 
 #include "arrow/buffer.h"
 #include "arrow/compute/exec.h"
+#include "arrow/device_allocation_type_set.h"
 #include "arrow/result.h"
 #include "arrow/type_traits.h"
 #include "arrow/util/bit_util.h"
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/hash_util.h"
-#include "arrow/util/logging.h"
+#include "arrow/util/logging_internal.h"
 #include "arrow/util/macros.h"
 
 namespace arrow {
@@ -75,7 +76,7 @@ Result<std::unique_ptr<KernelState>> ScalarAggregateKernel::MergeAll(
   for (auto& state : states) {
     RETURN_NOT_OK(kernel->merge(ctx, std::move(*state), out.get()));
   }
-  return std::move(out);
+  return out;
 }
 
 // ----------------------------------------------------------------------
@@ -361,7 +362,8 @@ size_t InputType::Hash() const {
     case InputType::EXACT_TYPE:
       hash_combine(result, type_->Hash());
       break;
-    default:
+    case InputType::ANY_TYPE:
+    case InputType::USE_TYPE_MATCHER:
       break;
   }
   return result;
@@ -378,10 +380,8 @@ std::string InputType::ToString() const {
       break;
     case InputType::USE_TYPE_MATCHER: {
       ss << type_matcher_->ToString();
-    } break;
-    default:
-      DCHECK(false);
       break;
+    }
   }
   return ss.str();
 }
@@ -400,9 +400,8 @@ bool InputType::Equals(const InputType& other) const {
       return type_->Equals(*other.type_);
     case InputType::USE_TYPE_MATCHER:
       return type_matcher_->Equals(*other.type_matcher_);
-    default:
-      return false;
   }
+  return false;
 }
 
 bool InputType::Matches(const DataType& type) const {
@@ -411,21 +410,23 @@ bool InputType::Matches(const DataType& type) const {
       return type_->Equals(type);
     case InputType::USE_TYPE_MATCHER:
       return type_matcher_->Matches(type);
-    default:
-      // ANY_TYPE
+    case InputType::ANY_TYPE:
       return true;
   }
+  return false;
 }
 
 bool InputType::Matches(const Datum& value) const {
   switch (value.kind()) {
+    case Datum::NONE:
+    case Datum::RECORD_BATCH:
+    case Datum::TABLE:
+      DCHECK(false) << "Matches expects ARRAY, CHUNKED_ARRAY or SCALAR";
+      return false;
     case Datum::ARRAY:
     case Datum::CHUNKED_ARRAY:
     case Datum::SCALAR:
       break;
-    default:
-      DCHECK(false);
-      return false;
   }
   return Matches(*value.type());
 }
@@ -445,11 +446,13 @@ const TypeMatcher& InputType::type_matcher() const {
 
 Result<TypeHolder> OutputType::Resolve(KernelContext* ctx,
                                        const std::vector<TypeHolder>& types) const {
-  if (kind_ == OutputType::FIXED) {
-    return type_.get();
-  } else {
-    return resolver_(ctx, types);
+  switch (kind_) {
+    case OutputType::FIXED:
+      return type_;
+    case OutputType::COMPUTED:
+      break;
   }
+  return resolver_(ctx, types);
 }
 
 const std::shared_ptr<DataType>& OutputType::type() const {
@@ -463,11 +466,13 @@ const OutputType::Resolver& OutputType::resolver() const {
 }
 
 std::string OutputType::ToString() const {
-  if (kind_ == OutputType::FIXED) {
-    return type_->ToString();
-  } else {
-    return "computed";
+  switch (kind_) {
+    case OutputType::FIXED:
+      return type_->ToString();
+    case OutputType::COMPUTED:
+      break;
   }
+  return "computed";
 }
 
 // ----------------------------------------------------------------------
