@@ -27,6 +27,7 @@
 #include "parquet/thrift_internal.h"
 
 #include "arrow/util/int_util_overflow.h"
+#include "arrow/util/logging_internal.h"
 #include "arrow/util/unreachable.h"
 
 #include <limits>
@@ -256,10 +257,11 @@ class RowGroupPageIndexReaderImpl : public RowGroupPageIndexReader {
     auto descr = row_group_metadata_->schema()->Column(i);
 
     // Get decryptor of column index if encrypted.
-    std::shared_ptr<Decryptor> decryptor = parquet::GetColumnMetaDecryptor(
-        col_chunk->crypto_metadata().get(), file_decryptor_);
+    std::unique_ptr<Decryptor> decryptor =
+        InternalFileDecryptor::GetColumnMetaDecryptorFactory(
+            file_decryptor_, col_chunk->crypto_metadata().get())();
     if (decryptor != nullptr) {
-      UpdateDecryptor(decryptor, row_group_ordinal_, /*column_ordinal=*/i,
+      UpdateDecryptor(decryptor.get(), row_group_ordinal_, /*column_ordinal=*/i,
                       encryption::kColumnIndex);
     }
 
@@ -295,10 +297,11 @@ class RowGroupPageIndexReaderImpl : public RowGroupPageIndexReader {
     uint32_t length = static_cast<uint32_t>(offset_index_location->length);
 
     // Get decryptor of offset index if encrypted.
-    std::shared_ptr<Decryptor> decryptor =
-        GetColumnMetaDecryptor(col_chunk->crypto_metadata().get(), file_decryptor_);
+    std::unique_ptr<Decryptor> decryptor =
+        InternalFileDecryptor::GetColumnMetaDecryptorFactory(
+            file_decryptor_, col_chunk->crypto_metadata().get())();
     if (decryptor != nullptr) {
-      UpdateDecryptor(decryptor, row_group_ordinal_, /*column_ordinal=*/i,
+      UpdateDecryptor(decryptor.get(), row_group_ordinal_, /*column_ordinal=*/i,
                       encryption::kOffsetIndex);
     }
 
